/**************************************************************
	16F1827 Small Volt-Amp Mater
	File-Name Small_VA_Mater_V1.c
	Gucci-Tech C.H
	MPLAB 8.60 /	Hitech PICC Pro Lite mode v9.80
	
	LED C-533SR White 3Digit Cathode-Common
	
	Digikey Remocon
	2011/5/30 PM 06:00 Coting Start
	
	Pin assignment:
	(1)RA2/AN2		Mode-Switch 10k-PullUp
	(2)RA3/AN3/Vref+ 2.5V-Vref LM385Z-2.5
	(3)RA4/AN4		7Seg-AnodeCommon DIG-3 xx1
	(4)RA5/MCLR		DispSpped-Switch 10k-PullUp
	(5)Vss
	(6)RB0/INT		7Seg-'a'segment
	(7)RB1/AN11 	7Seg-'b'segment
	(8)RB2/AN/10	7Seg-'c'segment
	(9)RB3/AN9/CCP1	7Seg-'d'segment
   (10)RB4/AN8		7Seg-'c'segment
   (11)RB5/AN7		7Seg-'d'segment
   (12)RB6/AN5/ICSPCLK 	7Seg-'e'segment
   (13)RB7/AN6/ICSPDAT	7Seg-'f'segment
   (14)VDD
   (15)RA6			7Seg-AnodeCommon DIG-2 x1x
   (16)RA7			7Seg-AnodeCommon DIG-1 1xx
   (17)RA0/AN0		Volt-Detect-IN
   (18)RA1/AN1		Amp-Detect-IN

**************************************************************/
#include <htc.h>
#pragma	jis

__CONFIG(	
        FOSC_INTOSC & WDTE_ON & PWRTE_OFF & MCLRE_OFF & CP_OFF 
        & CPD_OFF & BOREN_OFF & CLKOUTEN_OFF & IESO_OFF & FCMEN_OFF
);

__CONFIG(
	WRT_OFF & PLLEN_OFF & STVREN_ON & BORV_19 & LVP_OFF
);

void AD_Conv_V(void);
void AD_Conv_A(void);
unsigned int GetData(unsigned char AD_Chan);
void Delay_10ms(unsigned char time);

#define MHz 000000
#define _XTAL_FREQ 4MHz

#define Mode_SW		RA2		// Volte/Ampere Mode-Switch
#define DSP_SP_SW	RA5		// Disp Speed Switch
#define Seg_Digt1	RA4		// Digit1 xx1
#define Seg_Digt2	RA6		// Digit2 x1x
#define Seg_Digt3	RA7		// Digit3 1xx

#define FS_Volt		25.0	// AN0 Full Scale 99.9[V]
#define FS_Amp		2.50	// AN1 Full Scale 2.00[A]


int const LED_SEG[18] = {
//	 DPgfedcba
	0b00111111,				/* '0' */
	0b00000110,				/* '1' */
	0b01011011,				/* '2' */
	0b01001111,				/* '3' */
	0b01100110,				/* '4' */
	0b01101101,				/* '5' */
	0b01111101,				/* '6' */
	0b00100111,				/* '7' */
	0b01111111,				/* '8' */
	0b01101111,				/* '9' */
	0b01110111,				/* 'A' */
	0b01111100,				/* 'b' */
	0b01011000,				/* 'c' */
	0b01011110,				/* 'd' */
	0b01111001,				/* 'E' */
	0b01110001,				/* 'F' */
	0b10000000,				/* 'DP' */
	0b11111111,				/* 'Blank' */
};	

unsigned char Digit;
unsigned char Mode_F;		// Mode Flag '0'=Voltage , '1'=Ampere
unsigned char Speed_F;
unsigned int LED_SegBF;
unsigned int ADtemp;
unsigned short long ADtemp2;
unsigned char DotPosi;

float Volt_Dat;
float Amp_Dat;

// Interrupt Routine //
void interrupt isr(void){
	// TMR2 Interrput //
	if(TMR2IF){
		TMR2 	= 131;					// (1/(4MHz/4))*16*(256-131) = 2ms
		switch(Digit){
			case 0:
				Digit++;
				LATA = LATA | 0xD0;		// ALL Segment 'OFF'
				LATB = LED_SEG[LED_SegBF/100];
				if(DotPosi == 3)
					LATB |= 0x80;
				Seg_Digt3 = 0;			// 7Seg-AnodeCommon4 'ON'
				break;
			case 1:
				Digit++;
				LATA = LATA | 0xD0;		// ALL Segment 'OFF'
				LATB = LED_SEG[(LED_SegBF%100)/10];
				if(DotPosi == 2)
					LATB |= 0x80;
				Seg_Digt2 = 0;			// 7Seg-AnodeCommon3 'ON'
				break;
			case 2:
				Digit = 0;
				LATA = LATA | 0xD0;		// ALL Segment 'OFF'
				LATB = LED_SEG[(LED_SegBF%100)%10];
				if(DotPosi == 1)
					LATB |= 0x80;
				Seg_Digt1 = 0;			// 7Seg-AnodeCommon2 'ON'
				break;
			default:
				break;
		}
		TMR2IF = 0;						// TMR2 Interrupt Flag Clear
	}
}

void AD_Conv_V(void){
	//***** Voltage Data A/D Convert *****//
	ADtemp = GetData(0x00);				// AN0 Voltage Sampling
	Volt_Dat = (ADtemp * FS_Volt)/1024;
	Volt_Dat += 0.005;
	Volt_Dat *= 10000;
	ADtemp2 = Volt_Dat;
	ADtemp2 = ADtemp2 / 1000;
}

void AD_Conv_A(void){
	//***** Ampere Data A/D Convert *****//
	ADtemp = GetData(0x04);				// AN1 Ampere Sampling
	Amp_Dat = (ADtemp * FS_Amp)/1024;
	Amp_Dat += 0.0005;
	Amp_Dat *= 10000;
	ADtemp2 = Amp_Dat;
	ADtemp2 = Amp_Dat / 100;
}

unsigned int GetData(unsigned char AD_Chan){
	ADON = 0;						// A/D Disable
	ADCON0 = AD_Chan;				// A/D Channel Select
	ADON = 1;						// A/D Enable
	__delay_us(20);					// Convert Time Wait 20us
	ADGO = 1;						// Convert START
	while(ADGO);
	return(ADRESH * 256 + ADRESL);
}

//Delay 100msec --------------------------------
void Delay_10ms(unsigned char time){
	while(time--){
		__delay_ms(10);
	}
}

//------------------ main Routine --------------------
void main(void){
//Initialization
	OSCCON	= 0b01101010; 			// PLL Disable, 4MHz internal Oscillator
	GIE		= 0;					// Interrupt service is disable
	PEIE	= 0;					// Peripheral Interrput Disable
	APFCON0	= 0b00100110;			// Rx/DT=RB1,SDO=RB2,SS=RA5,P2B=RB7,
 									// CCP2=RB6,P1D=RA6,P1C=RA7,CCP1=RB3
 	APFCON1	= 1;					// Tx/CK=RB5

 	ANSELA	= 0b00001011;			// ANA0,AN1,AN3=Analog-IN,Vref+
 	ANSELB	= 0b00000000;			// PORTB ALL Digital
	ADCON1	= 0b10100010;			// AD-Data=Right_Justified,A/D-Clock=Fosc/32
									// Vref-=AVss Conected,Vref+=External Vref+
//  		bit 76543210
	WDTCON	= 0b00011001;			// Prescale=1:131072(Intervl 4s),SWDTEN=1
									// (1/31kHz) * 131072 = 4.22813s
	OPTION_REG = 0b00000111;		// WPUEN=Enable,ClockSource=Internal(Fosc/4)
									// PSA=TMR0,Prescaler=1/256
	TMR0 	= 0;					// (1/(4MHz/4))*256*256 = 65.536ms
	
	T2CON	= 0b00000010;			// Postscaler=1:1,T2-ON/OFF=OFF,Prescaler=1/16
	TMR2	= 131;					// (1/(4MHz/4))*16*(256-131) = 2ms

    TRISA 	= 0b00101111;			// PORTA I/O dirction setting
	WPUA	= 0b00100100;			// Pulll-up Enable Select
    PORTA	= 0b11110100;			// PORTA I/O initialize
	LATA	= 0b11110100;
	WPUA5	= 1;					// MCLR(RA5)(4Pin)=Pull-Up Enable

    TRISB 	= 0b00000000;			// PORTB I/O dirction setting
	WPUB	= 0b00000000;			// Pulll-up ALL-Disable Select
    PORTB	= 0b00000000;			// PORTB I/O initialize
	LATB	= 0b00000000;

	TMR0IF	= 0;					// TMR0 Interrupt Flag Clear
	TMR2IF	= 0;					// TMR2 Interrupt Flag Clear
	
	Digit = 0;
	LED_SegBF = 0;
	ADtemp = 0;
	ADtemp2 = 0;
	Volt_Dat = 0;
	Amp_Dat = 0;

	// Interrupt Enable //
	TMR2IE	= 1;					// TMR2 Interrput Enable
	PEIE	= 1;					// Peripheral Interrput Enable
	GIE		= 1;					// Global Interrupt Enable

	if(DSP_SP_SW == 1){
		Speed_F = 1;
		asm("clrwdt");				// WDT Clear
		WDTCON	= 0b00010111;		// Prescale=1:65536(Intervl 2.114s),SWDTEN=1
	}else{
		Speed_F = 0;
		asm("clrwdt");				// WDT Clear
		WDTCON	= 0b00010101;		// Prescale=1:32768(Intervl 1.057s),SWDTEN=1
	}


	while(1){
		if((DSP_SP_SW == 1) && (Speed_F == 0)){
			Speed_F = 1;
			asm("clrwdt");					// WDT Clear
//			WDTCON	= 0b00011001;			// Prescale=1:131072(Intervl 4.23s),SWDTEN=1
			WDTCON	= 0b00010111;			// Prescale=1:65536(Intervl 2.114s),SWDTEN=1
		}
		if((DSP_SP_SW == 0) && (Speed_F == 1)){
			Speed_F = 0;
			asm("clrwdt");					// WDT Clear
//			WDTCON	= 0b00010111;			// Prescale=1:65536(Intervl 2.114s),SWDTEN=1
			WDTCON	= 0b00010101;			// Prescale=1:32768(Intervl 1.057s),SWDTEN=1
		}

		if(Mode_SW == 1){
			//*** Voltage Data A/D Convert ***//
			Mode_F = 0;						// Mode Flag '0' Set
			AD_Conv_V();
			DotPosi = 2;
			LED_SegBF = ADtemp2;
			TMR2ON	= 1;					// TMR2 START
		}else{
			//*** Ampere Data A/D Convert ***//
			Mode_F = 1;						// Mode Flag '1' Set
			AD_Conv_A();
			DotPosi = 3;
			LED_SegBF = ADtemp2;
			TMR2ON	= 1;					// TMR2 START
		}
		Delay_10ms(25);						// Display Cycle Time 0.25sec
		TMR2ON	= 0;						// TMR2 Stop
		PORTA = PORTA | 0xD0;				// ALL Segment 'OFF'
		PORTB = 0b00000000;

		asm("sleep");
		asm("nop");
	}
}